import stable from "stable";
import specificity from "csso/lib/restructure/prepare/specificity";
import csstree from "css-tree";
const List = csstree.List;
function flattenToSelectors(cssAst) {
  const selectors = [];
  csstree.walk(cssAst, {
    visit: "Rule",
    enter: function(node) {
      if ("Rule" !== node.type) return;
      const atrule = this.atrule;
      const rule = node;
      node.prelude.children.each((function(selectorNode, selectorItem) {
        const selector = {
          item: selectorItem,
          atrule,
          rule,
          pseudos: []
        };
        selectorNode.children.each((function(selectorChildNode, selectorChildItem, selectorChildList) {
          "PseudoClassSelector" !== selectorChildNode.type && "PseudoElementSelector" !== selectorChildNode.type || selector.pseudos.push({
            item: selectorChildItem,
            list: selectorChildList
          });
        }));
        selectors.push(selector);
      }));
    }
  });
  return selectors;
}
function filterByMqs(selectors, useMqs) {
  return selectors.filter((function(selector) {
    if (null === selector.atrule) return ~useMqs.indexOf("");
    const mqName = selector.atrule.name;
    let mqStr = mqName;
    selector.atrule.expression && "MediaQueryList" === selector.atrule.expression.children.first().type && (mqStr = [ mqName, csstree.generate(selector.atrule.expression) ].join(" "));
    return ~useMqs.indexOf(mqStr);
  }));
}
function filterByPseudos(selectors, usePseudos) {
  return selectors.filter((function(selector) {
    const pseudoSelectorsStr = csstree.generate({
      type: "Selector",
      children: (new List).fromArray(selector.pseudos.map((function(pseudo) {
        return pseudo.item.data;
      })))
    });
    return ~usePseudos.indexOf(pseudoSelectorsStr);
  }));
}
function cleanPseudos(selectors) {
  selectors.forEach((function(selector) {
    selector.pseudos.forEach((function(pseudo) {
      pseudo.list.remove(pseudo.item);
    }));
  }));
}
function compareSpecificity(aSpecificity, bSpecificity) {
  for (let i = 0; i < 4; i += 1) {
    if (aSpecificity[i] < bSpecificity[i]) return -1;
    if (aSpecificity[i] > bSpecificity[i]) return 1;
  }
  return 0;
}
function compareSimpleSelectorNode(aSimpleSelectorNode, bSimpleSelectorNode) {
  return compareSpecificity(specificity(aSimpleSelectorNode), specificity(bSimpleSelectorNode));
}
function _bySelectorSpecificity(selectorA, selectorB) {
  return compareSimpleSelectorNode(selectorA.item.data, selectorB.item.data);
}
function sortSelectors(selectors) {
  return stable(selectors, _bySelectorSpecificity);
}
function csstreeToStyleDeclaration(declaration) {
  return {
    name: declaration.property,
    value: csstree.generate(declaration.value),
    priority: declaration.important ? "important" : ""
  };
}
function getCssStr(elem) {
  return elem.content[0].text || elem.content[0].cdata || [];
}
function setCssStr(elem, css) {
  if (elem.content[0].cdata) {
    elem.content[0].cdata = css;
    return elem.content[0].cdata;
  }
  elem.content[0].text = css;
  return elem.content[0].text;
}
export { flattenToSelectors, filterByMqs, filterByPseudos, cleanPseudos, compareSpecificity, compareSimpleSelectorNode, sortSelectors, csstreeToStyleDeclaration, getCssStr, setCssStr };