import { removeLeadingZero } from "../lib/svgo/tools";
const name = "cleanupListOfValues";
const type = "perItem";
const active = !1;
const description = "rounds list of values to the fixed precision";
const params = {
  floatPrecision: 3,
  leadingZero: !0,
  defaultPx: !0,
  convertToPx: !0
};
const regNumericValues = /^([-+]?\d*\.?\d+([eE][-+]?\d+)?)(px|pt|pc|mm|cm|m|in|ft|em|ex|%)?$/;
const regSeparator = /\s+,?\s*|,\s*/;
const absoluteLengths = {
  cm: 96 / 2.54,
  mm: 96 / 25.4,
  in: 96,
  pt: 4 / 3,
  pc: 16
};
const fn = function(item, params) {
  item.hasAttr("points") && roundValues(item.attrs.points);
  item.hasAttr("enable-background") && roundValues(item.attrs["enable-background"]);
  item.hasAttr("viewBox") && roundValues(item.attrs.viewBox);
  item.hasAttr("stroke-dasharray") && roundValues(item.attrs["stroke-dasharray"]);
  item.hasAttr("dx") && roundValues(item.attrs.dx);
  item.hasAttr("dy") && roundValues(item.attrs.dy);
  item.hasAttr("x") && roundValues(item.attrs.x);
  item.hasAttr("y") && roundValues(item.attrs.y);
  function roundValues($prop) {
    let num;
    let units;
    let match;
    let matchNew;
    const listsArr = $prop.value.split(regSeparator);
    const roundedListArr = [];
    listsArr.forEach((function(elem) {
      match = elem.match(regNumericValues);
      matchNew = elem.match(/new/);
      if (match) {
        num = +(+match[1]).toFixed(params.floatPrecision);
        units = match[3] || "";
        if (params.convertToPx && units && units in absoluteLengths) {
          const pxNum = +(absoluteLengths[units] * match[1]).toFixed(params.floatPrecision);
          if (String(pxNum).length < match[0].length) {
            num = pxNum;
            units = "px";
          }
        }
        params.leadingZero && (num = removeLeadingZero(num));
        params.defaultPx && "px" === units && (units = "");
        roundedListArr.push(num + units);
      } else matchNew ? roundedListArr.push("new") : elem && roundedListArr.push(elem);
    }));
    const roundedList = roundedListArr.join(" ");
    $prop.value = roundedList;
  }
};
export { name, type, active, description, params, fn };