import csstree from "css-tree";
import { flattenToSelectors, filterByMqs, filterByPseudos, cleanPseudos, sortSelectors, csstreeToStyleDeclaration, getCssStr, setCssStr } from "../lib/css-tools";
const name = "inlineStyles";
const type = "full";
const active = !0;
const params = {
  onlyMatchedOnce: !0,
  removeMatchedSelectors: !0,
  useMqs: [ "", "screen" ],
  usePseudos: [ "" ]
};
const description = "inline styles (additional options)";
const fn = function(document, opts) {
  const styleEls = document.querySelectorAll("style");
  if (null === styleEls) return document;
  const styles = [];
  let selectors = [];
  for (const styleEl of styleEls) {
    if (styleEl.isEmpty() || styleEl.closestElem("foreignObject")) continue;
    const cssStr = getCssStr(styleEl);
    let cssAst = {};
    try {
      cssAst = csstree.parse(cssStr, {
        parseValue: !1,
        parseCustomProperty: !1
      });
    } catch (parseError) {
      continue;
    }
    styles.push({
      styleEl,
      cssAst
    });
    selectors = selectors.concat(flattenToSelectors(cssAst));
  }
  const selectorsMq = filterByMqs(selectors, opts.useMqs);
  const selectorsPseudo = filterByPseudos(selectorsMq, opts.usePseudos);
  cleanPseudos(selectorsPseudo);
  const sortedSelectors = sortSelectors(selectorsPseudo).reverse();
  let selector;
  let selectedEl;
  for (selector of sortedSelectors) {
    const selectorStr = csstree.generate(selector.item.data);
    let selectedEls = null;
    try {
      selectedEls = document.querySelectorAll(selectorStr);
    } catch (selectError) {
      if (selectError.message.includes("Unmatched selector:")) continue;
      throw selectError;
    }
    null !== selectedEls && (selector.selectedEls = selectedEls);
  }
  for (selector of sortedSelectors) if (selector.selectedEls && !(opts.onlyMatchedOnce && null !== selector.selectedEls && selector.selectedEls.length > 1)) {
    for (selectedEl of selector.selectedEls) null !== selector.rule && csstree.walk(selector.rule, {
      visit: "Declaration",
      enter: function(styleCsstreeDeclaration) {
        const styleDeclaration = csstreeToStyleDeclaration(styleCsstreeDeclaration);
        null !== selectedEl.style.getPropertyValue(styleDeclaration.name) && selectedEl.style.getPropertyPriority(styleDeclaration.name) >= styleDeclaration.priority || selectedEl.style.setProperty(styleDeclaration.name, styleDeclaration.value, styleDeclaration.priority);
      }
    });
    opts.removeMatchedSelectors && null !== selector.selectedEls && selector.selectedEls.length > 0 && selector.rule.prelude.children.remove(selector.item);
  }
  if (!opts.removeMatchedSelectors) return document;
  for (selector of sortedSelectors) if (selector.selectedEls && !(opts.onlyMatchedOnce && null !== selector.selectedEls && selector.selectedEls.length > 1)) for (selectedEl of selector.selectedEls) {
    const firstSubSelector = selector.item.data.children.first();
    "ClassSelector" === firstSubSelector.type && selectedEl.class.remove(firstSubSelector.name);
    void 0 === selectedEl.class.item(0) && selectedEl.removeAttr("class");
    "IdSelector" === firstSubSelector.type && selectedEl.removeAttr("id", firstSubSelector.name);
  }
  for (const style of styles) {
    csstree.walk(style.cssAst, {
      visit: "Rule",
      enter: function(node, item, list) {
        ("Atrule" === node.type && null !== node.block && node.block.children.isEmpty() || "Rule" === node.type && node.prelude.children.isEmpty()) && list.remove(item);
      }
    });
    if (style.cssAst.children.isEmpty()) {
      const styleParentEl = style.styleEl.parentNode;
      styleParentEl.spliceContent(styleParentEl.content.indexOf(style.styleEl), 1);
      if ("defs" === styleParentEl.elem && 0 === styleParentEl.content.length) {
        const defsParentEl = styleParentEl.parentNode;
        defsParentEl.spliceContent(defsParentEl.content.indexOf(styleParentEl), 1);
      }
    } else setCssStr(style.styleEl, csstree.generate(style.cssAst));
  }
  return document;
};
export { name, type, active, description, params, fn };